import json
import pkg_resources

# list of vallinam,mellinam,idaiyinam
vallinam = ["க", "ச", "ட", "த", "ப", "ற"]
mellinam = ["ங", "ஞ", "ண", "ந", "ம", "ன"]
idaiyinam = ["ய", "ர", "ல", "வ", "ழ", "ள"]

# Phonetic pronouncation
json_file_path = pkg_resources.resource_filename(__name__, "translit_rules.json")
with open(json_file_path, "r", encoding="utf-8") as file:
    transliteration_rules = json.load(file)

ka_rule = transliteration_rules.get("ka_rule", {})
sa_rule = transliteration_rules.get("sa_rule", {})
ta_rule = transliteration_rules.get("ta_rule", {})
pa_rule = transliteration_rules.get("pa_rule", {})
tha_rule = transliteration_rules.get("tha_rule", {})
extras = transliteration_rules.get("extras", {})


# Combines constanant with vowel sign
def extras_comb(letter):
    return [letter + const for const in extras]


# Rules
def olipeyarppu(word_start, tam_word, prev_comb, next_next_char, next_char):
    roman_text = ""
    if word_start:
        if tam_word in extras_comb("க"):
            roman_text += ka_rule.get(tam_word, {})

        elif tam_word in extras_comb("த"):
            roman_text += tha_rule.get(tam_word, {})

        elif tam_word in extras_comb("ட")[2:]:
            roman_text += ta_rule.get(tam_word, {})

        elif tam_word in extras_comb("ப"):
            roman_text += pa_rule.get(tam_word, {})

        else:
            roman_text += transliteration_rules["letter_rule"].get(tam_word, {})

    elif tam_word == "க" and prev_comb == "று":
        roman_text += "ka"

    elif prev_comb in [
        letter + "்" for letter in vallinam + idaiyinam + mellinam + ["ஸ", "ஷ"]
    ]:

        if prev_comb == "ன்":
            if tam_word == "று":
                roman_text += "dru"
            elif tam_word == "றி":
                roman_text += "dri"
            elif tam_word == "ற":
                roman_text += "dra"
            elif tam_word == "றை":
                roman_text += "drai"
            elif tam_word in extras_comb("ப"):
                roman_text += transliteration_rules["letter_rule"].get(tam_word, "")

        if roman_text == "" and prev_comb in ["க்", "ச்", "ட்", "த்", "ப்", "ற்", "ஷ்", "ஸ்"]:

            roman_text += ka_rule.get(tam_word, "")
            roman_text += sa_rule.get(tam_word, "")
            roman_text += ta_rule.get(tam_word, "")
            roman_text += tha_rule.get(tam_word, "")
            roman_text += pa_rule.get(tam_word, "")

        if prev_comb in ["ஞ்", "ங்"]:
            for key, value in extras.items():
                if tam_word in [letter + key for letter in vallinam]:
                    roman_text += value

        if roman_text == "":
            roman_text += transliteration_rules["letter_rule"].get(tam_word, "")

    elif tam_word == "ற்" and next_next_char == "ற":
        roman_text += "t"

    else:
        roman_text += transliteration_rules["letter_rule"].get(tam_word, "")

    return roman_text



import json
import re
import pkg_resources

from .olipeyarppu import olipeyarppu
import pkg_resources

json_file_path = pkg_resources.resource_filename(__name__, "translit_rules.json")

# Open the file with the absolute path
with open(json_file_path, "r", encoding="utf-8") as file:
    transliteration_rules = json.load(file)

# Some special cases
special_case = transliteration_rules.get("special_case", {})


# Replace the tam_word with special case word
def replace_word(match):
    word = match.group(0)
    return special_case.get(word, "")


# Main function for transliteration
def translite(tamil_text):

    roman_text = ""
    word_start = True
    i = 0

    # Identifying patter for special case
    pattern = r"|".join(re.escape(key) for key in special_case.keys())
    tamil_text = re.sub(pattern, replace_word, tamil_text)
    while i < len(tamil_text):
        if (
            i + 1 < len(tamil_text)
            and tamil_text[i : i + 2] in transliteration_rules["letter_rule"]
        ):
            ########### const+vowels (like - கி, மீ) ###########

            tam_word = tamil_text[i : i + 2] if i + 1 < len(tamil_text) else ""
            prev_comb = tamil_text[i - 2 : i] if i - 2 >= 0 else ""
            next_next_char = tamil_text[i + 2] if i + 2 < len(tamil_text) else ""
            next_char = tamil_text[i + 1] if i + 1 < len(tamil_text) else ""

            roman_text += olipeyarppu(
                word_start, tam_word, prev_comb, next_next_char, next_char
            )

            i += 1  # Skip the next character as it's already processed

            word_start = False  # After processing the first character, it's no longer at the letter_rule

        ########### Handle single characters like அ, த ###########

        elif tamil_text[i] in transliteration_rules["letter_rule"]:
            tam_word = tamil_text[i]
            next_char = tamil_text[i + 1] if i + 1 < len(tamil_text) else ""
            prev_comb = tamil_text[i - 2 : i] if i - 2 >= 0 else ""
            next_next_char = tamil_text[i + 2] if i + 2 < len(tamil_text) else ""
            roman_text += olipeyarppu(
                word_start, tam_word, prev_comb, next_next_char, next_char
            )

        else:
            roman_text += tamil_text[
                i
            ]  # If no mapping exists, keep the character as it is

        i += 1
        word_start = False

        # Handle word boundary detection by checking if the next character is a space
        if i < len(tamil_text) and tamil_text[i - 1] in [
            " ",
            "\n",
            ",",
            ";",
            ":",
            "'",
            "-",
            "_",
            "(",
            ")",
            ".",
            "$",
            "@",
            "#",
            "%",
            "*",
        ]:
            word_start = True  # Reset word letter_rule for next word

    return roman_text.strip()



from setuptools import setup, find_packages

setup(
    name="tamil_translite",
    version="0.2.6",
    description="A package for phonetic transliteration of Tamil text into English.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    author="Harikaran Saravanan",
    author_email="irahnarak@gmail.com",
    url="https://github.com/irahnarak/tamil_translite",
    packages=find_packages(),
    package_data={
        "tamil_translite": ["translit_rules.json"],  # Include the .json file
    },
    include_package_data=True,
    tests_require=["pytest"],
    keywords="Tamil, Transliteration, Phonetic, Tamil to English",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.9",
)



import pytest
from tamil_translite import translite


def translite_func(sentence):
    return translite(sentence)


def test_translite():
    assert (
        translite(
            """ஓர் ஊர்தியில் ஏறி, கீழே இறங்கி சூடான சுத்தமான நெய் உணவு ஆவினை எடுத்து, திடீரென ஐந்நூறு கூடை குண்டூசியும் ஈரேழு ஔடத மின்னூல்களும் அனுப்பி ஞாபகமூட்டியது ஒரு தூதுக்குழு"""
        )
        == "or oordhiyil yeri, keezhae irangi soodaana suththamaana nei unavu aavinai eduththu, thideerena ainnooru koodai kundoosiyum eeraezhu audadha minnoolgalum anuppi njaabagamoottiyadhu oru thoodhukkuzhu"
    )


@pytest.mark.parametrize(
    "input_word, expected_value",
    [
        ("காக்கா", "kaakkaa"),
        ("அகம்", "agam"),
    ],
)
def test_ka_ga_variation(input_word, expected_value):
    assert translite(input_word) == expected_value


@pytest.mark.parametrize(
    "input_word, expected_value",
    [
        ("சச்சரவு", "sachcharavu"),
        ("பசலை", "pasalai"),
    ],
)
def test_sa_cha_variation(input_word, expected_value):
    assert translite(input_word) == expected_value


@pytest.mark.parametrize(
    "input_word, expected_value",
    [("சம்மட்டி", "sammatti"), ("படி", "padi"), ("டபரா", "dabaraa")],
)
def test_ta_da_variation(input_word, expected_value):
    assert translite(input_word) == expected_value


@pytest.mark.parametrize(
    "input_word, expected_value",
    [
        ("பதம்", "padham"),
        ("சத்தம்", "saththam"),
        ("துணை", "thunai"),
    ],
)
def test_tha_dha_variation(input_word, expected_value):
    assert translite(input_word) == expected_value


@pytest.mark.parametrize(
    "input_word, expected_value",
    [
        ("பாப்பா", "paappaa"),
        ("செண்பகம்", "senbagam"),
    ],
)
def test_pa_ba_variation(input_word, expected_value):
    assert translite(input_word) == expected_value
